@@ -35,7 +35,7 @@ static int    num_flags_processed = 0;
 
 extern const char *minor_str[];
 
-#define VERSION "v9.37"
+#define VERSION "v9.38"
 
 #ifndef O_DIRECT
 #define O_DIRECT	040000	/* direct disk access, not easily obtained from headers */
@@ -55,7 +55,7 @@ static int do_defaults = 0, do_flush = 0, do_ctimings, do_timings = 0;
 static int do_identity = 0, get_geom = 0, noisy = 1, quiet = 0;
 static int do_flush_wcache = 0;
 
-//static int set_wdidle3  = 0;
+static int set_wdidle3  = 0, get_wdidle3 = 0, wdidle3 = 0;
 static int   set_timings_offset = 0;
 static __u64 timings_offset = 0;
 static int set_fsreadahead= 0, get_fsreadahead= 0, fsreadahead= 0;
@@ -706,15 +706,25 @@ static unsigned int get_erase_timeout_secs (int fd, int enhanced)
 	if (id) {
 		timeout = id[idx];
 		if (timeout && timeout <= 0xff) {
-			if (timeout == 0xff)
+			/*
+			 * 0xff means "more than 254 2-minute intervals (508+ minutes),
+			 * but we really want a better idea than that.
+			 * Norman Diamond suggests allowing 1sec per 30MB of capacity.
+			 */
+			if (timeout == 0xff) {
+				__u64 lba_limit = get_lba_capacity(id);
+				__u64 estimate = (lba_limit / 2048ULL) / 30ULL / 60;
 				timeout = 508 + 60;  /* spec says > 508 minutes */
-			else
+				if (timeout < estimate)
+					timeout = estimate;
+			} else {
 				timeout = (timeout * 2) + 5;  /* Add on a 5min margin */
+			}
 		}
 	}
 	if (!timeout)
 		timeout = 2 * 60;  /* default: two hours */
-	timeout *= 60; /* secs */
+	timeout *= 60; /* convert minutes to seconds */
 	return timeout;
 }
 
@@ -778,7 +788,7 @@ do_set_security (int fd)
 			exit(EINVAL);
 	}
 	printf(" Issuing %s command, password=\"%s\", user=%s",
-		description, security_password, data[0] ? "master" : "user");
+		description, security_password, (data[0] & 1) ? "master" : "user");
 	if (security_command == ATA_OP_SECURITY_SET_PASS)
 		printf(", mode=%s", data[1] ? "max" : "high");
 	printf("\n");
@@ -949,44 +959,6 @@ static int abort_if_not_full_device (int fd, __u64 lba, const char *devname, con
 	exit(EINVAL);
 }
 
-#if 0
-static int do_wdidle3 (int fd, const char *devname)
-{
-	struct ata_tf tf;
-	int err = 0;
-	const unsigned char vu_op = 0x8a;
-
-	abort_if_not_full_device(fd, 0, devname, NULL);
-	confirm_please_destroy_my_drive("--wdidle3", "This is not fully implemented yet, and could destroy the drive and/or all data on it.");
-	printf("attempting to tweak Western Digital \"idle-3\" parameters\n");
-	fflush(stdout);
-
-  {
-    unsigned char bits;
-    for (bits = 0; bits <= 0x1f; ++bits) {
-	tf_init(&tf, vu_op, 0, 0);
-	tf.lob.feat  = 'W';
-	tf.lob.nsect = 'D';
-	tf.lob.lbal  = 'C';
-	tf.lob.lbam  = 0x00;
-	tf.lob.lbah  = 0x00;
-	tf.dev       = 0xa0 | (tf.dev & 0xb0) | (bits & 0xf);
-	if (bits >= 0x10)
-		tf.dev |= 0x40;
-
-	/* This probably wants to transfer data, but.. ???? */
-	if (sg16(fd, SG_WRITE, SG_PIO, &tf, NULL, 0, 5 /* seconds */)) {
-		err = errno;
-		perror("FAILED");
-	} else {
-		printf("succeeded\n");
-	}
-    }
-  }
-	return err;
-}
-#endif
-
 static __u16 *get_dco_identify_data (int fd, int quietly)
 {
 	static __u8 args[4+512];
@@ -1016,6 +988,9 @@ static __u64 do_get_native_max_sectors (int fd)
 	__u64 max = 0;
 	struct hdio_taskfile r;
 
+	get_identify_data(fd);
+	if (!id)
+		exit(EIO);
 	memset(&r, 0, sizeof(r));
 	r.cmd_req = TASKFILE_CMD_REQ_NODATA;
 	r.dphase  = TASKFILE_DPHASE_NONE;
@@ -1027,9 +1002,6 @@ static __u64 do_get_native_max_sectors (int fd)
 	r.iflags.lob.lbah     = 1;
 	r.lob.dev = 0x40;
 
-	get_identify_data(fd);
-	if (!id)
-		exit(EIO);
 	if (((id[83] & 0xc400) == 0x4400) && (id[86] & 0x0400)) {
 		r.iflags.hob.lbal  = 1;
 		r.iflags.hob.lbam  = 1;
@@ -1466,6 +1438,7 @@ static void usage_help (int clue, int rc)
 	" -H   Read temperature from drive (Hitachi only)\n"
 	" -i   Display drive identification\n"
 	" -I   Detailed/current information directly from drive\n"
+	" -J   Get/set Western DIgital \"Idle3\" timeout for a WDC \"Green\" drive (DANGEROUS)\n"
 	" -k   Get/set keep_settings_over_reset flag (0/1)\n"
 	" -K   Set drive keep_features_over_reset flag (0/1)\n"
 	" -L   Set drive doorlock (0/1) (removable harddisks only)\n"
@@ -1518,7 +1491,6 @@ static void usage_help (int clue, int rc)
 	" --trim-sector-ranges        Tell SSD firmware to discard unneeded data sectors: lba:count ..\n"
 	" --trim-sector-ranges-stdin  Same as above, but reads lba:count pairs from stdin\n"
 	" --verbose         Display extra diagnostics from some commands\n"
-	//" --wdidle3         Issue the Western Digitial \"Idle3\" command (EXTREMELY DANGEROUS)\n"
 	" --write-sector    Repair/overwrite a (possibly bad) sector directly on the media (VERY DANGEROUS)\n"
 	"\n");
 	exit(rc);
@@ -1578,8 +1550,16 @@ void process_dev (char *devname)
 		exit(do_trim_from_stdin(fd, devname));
 	}
 
-	//if (set_wdidle3)
-	//	do_wdidle3(fd, devname);
+	if (set_wdidle3) {
+		unsigned char timeout = wdidle3_msecs_to_timeout(wdidle3);
+		confirm_please_destroy_my_drive("-J", "This implementation is not as thorough as the official WDIDLE3.EXE. Use at your own risk!");
+		if (get_wdidle3) {
+			printf(" setting wdidle3 to ");
+			wdidle3_print_timeout(timeout);
+			putchar('\n');
+		}
+		err = wdidle3_set_timeout(fd, timeout);
+	}
 	if (set_fsreadahead) {
 		if (get_fsreadahead)
 			printf(" setting fs readahead to %d\n", fsreadahead);
@@ -2149,6 +2129,15 @@ void process_dev (char *devname)
 				printf("%lld\n", start_lba);
 		}
 	}
+	if (get_wdidle3) {
+		unsigned char timeout = 0;
+		err = wdidle3_get_timeout(fd, &timeout);
+		if (!err) {
+			printf(" wdidle3      = ");
+			wdidle3_print_timeout(timeout);
+			putchar('\n');
+		}
+	}
 	if (get_powermode) {
 		__u8 args[4] = {ATA_OP_CHECKPOWERMODE1,0,0,0};
 		const char *state = "unknown";
@@ -2670,8 +2659,6 @@ get_longarg (void)
 	} else if (0 == strcasecmp(name, "read-sector")) {
 		read_sector = 1;
 		get_u64_parm(0, 0, NULL, &read_sector_addr, 0, lba_limit, name, lba_emsg);
-	//} else if (0 == strcasecmp(name, "wdidle3")) {
-	//	set_wdidle3 = 1;
 	} else if (0 == strcasecmp(name, "Istdout")) {
 		do_IDentity = 2;
 	} else if (0 == strcasecmp(name, "security-mode")) {
@@ -2759,6 +2746,7 @@ int main (int _argc, char **_argv)
 				case     SET_FLAG('H',hitachi_temp);
 				case      DO_FLAG('i',do_identity);
 				case      DO_FLAG('I',do_IDentity);
+				case GET_SET_PARM('J',"WDC-idle3-timeout",wdidle3,0,300);
 				case GET_SET_PARM('k',"kernel-keep-settings",keep,0,1);
 				case     SET_PARM('K',"drive-keep-settings",dkeep,0,1);
 				case     SET_PARM('L',"door-lock",doorlock,0,1);
